package aoc2015

import println
import readInput

/*
--- Day 2: I Was Told There Would Be No Math ---
The elves are running low on wrapping paper, and so they need to submit an order for more.  They have a list of the dimensions (length l, width w, and height h) of each present, and only want to order exactly as much as they need.
Fortunately, every present is a box (a perfect right rectangular prism), which makes calculating the required wrapping paper for each gift a little easier: find the surface area of the box, which is 2*l*w + 2*w*h + 2*h*l.  The elves also need a little extra paper for each present: the area of the smallest side.
For example:

A present with dimensions 2x3x4 requires 2*6 + 2*12 + 2*8 = 52 square feet of wrapping paper plus 6 square feet of slack, for a total of 58 square feet.
A present with dimensions 1x1x10 requires 2*1 + 2*10 + 2*10 = 42 square feet of wrapping paper plus 1 square foot of slack, for a total of 43 square feet.

All numbers in the elves' list are in feet.  How many total square feet of wrapping paper should they order?

*/
fun main() {

    val inlineTestInput = """
2x3x4
1x1x10
"""

    fun part1(input: List<String>): Int {
        return input.sumOf {
            val dims = it.split("x").map(String::toInt).sorted()
            2 * dims[0] * dims[1] + 2 * dims[1] * dims[2] + 2 * dims[0] * dims[2] + dims[0] * dims[1]
        }
    }

    fun part2(input: List<String>): Int {
        return input.sumOf {
            val dims = it.split("x").map(String::toInt).sorted()
            2 * (dims[0] + dims[1]) + dims.reduce(Int::times)
        }
    }

    // test if implementation meets criteria from the description, like:
    val testInput = inlineTestInput.trim().reader().readLines()
    //val testInput = readInput("aoc2015/Day02_test")
    val testInputPart1Result = part1(testInput)
    println("Part 1 Test: $testInputPart1Result")
    val testInputPart2Result = part2(testInput)
    println("Part 2 Test: $testInputPart2Result")
    check(testInputPart1Result == 58 + 43)
    check(testInputPart2Result == 34 + 14)

    val input = readInput("aoc2015/Day02")
    part1(input).println()
    part2(input).println()
}
