package aoc2023

import CharGrid
import println
import readInput
import kotlin.math.abs

/*
--- Day 11: Cosmic Expansion ---
You continue following signs for "Hot Springs" and eventually come across an observatory. The Elf within turns out to be a researcher studying cosmic expansion using the giant telescope here.
He doesn't know anything about the missing machine parts; he's only visiting for this research project. However, he confirms that the hot springs are the next-closest area likely to have people; he'll even take you straight there once he's done with today's observation analysis.
Maybe you can help him with the analysis to speed things up?
The researcher has collected a bunch of data and compiled the data into a single giant image (your puzzle input). The image includes empty space (.) and galaxies (#). For example:
...#......
.......#..
#.........
..........
......#...
.#........
.........#
..........
.......#..
#...#.....

The researcher is trying to figure out the sum of the lengths of the shortest path between every pair of galaxies. However, there's a catch: the universe expanded in the time it took the light from those galaxies to reach the observatory.
Due to something involving gravitational effects, only some space expands. In fact, the result is that any rows or columns that contain no galaxies should all actually be twice as big.
In the above example, three columns and two rows contain no galaxies:
   v  v  v
 ...#......
 .......#..
 #.........
>..........<
 ......#...
 .#........
 .........#
>..........<
 .......#..
 #...#.....
   ^  ^  ^

These rows and columns need to be twice as big; the result of cosmic expansion therefore looks like this:
....#........
.........#...
#............
.............
.............
........#....
.#...........
............#
.............
.............
.........#...
#....#.......

Equipped with this expanded universe, the shortest path between every pair of galaxies can be found. It can help to assign every galaxy a unique number:
....1........
.........2...
3............
.............
.............
........4....
.5...........
............6
.............
.............
.........7...
8....9.......

In these 9 galaxies, there are 36 pairs. Only count each pair once; order within the pair doesn't matter. For each pair, find any shortest path between the two galaxies using only steps that move up, down, left, or right exactly one . or # at a time. (The shortest path between two galaxies is allowed to pass through another galaxy.)
For example, here is one of the shortest paths between galaxies 5 and 9:
....1........
.........2...
3............
.............
.............
........4....
.5...........
.##.........6
..##.........
...##........
....##...7...
8....9.......

This path has length 9 because it takes a minimum of nine steps to get from galaxy 5 to galaxy 9 (the eight locations marked # plus the step onto galaxy 9 itself). Here are some other example shortest path lengths:

Between galaxy 1 and galaxy 7: 15
Between galaxy 3 and galaxy 6: 17
Between galaxy 8 and galaxy 9: 5

In this example, after expanding the universe, the sum of the shortest path between all 36 pairs of galaxies is 374.
Expand the universe, then find the length of the shortest path between every pair of galaxies. What is the sum of these lengths?

*/
fun main() {

    val inlineTestInput = """
...#......
.......#..
#.........
..........
......#...
.#........
.........#
..........
.......#..
#...#.....
"""

    fun parts(input: List<String>, exp: Int): Long {
        val image = CharGrid(input, '.')
        val exCol = IntArray(image.width)
        val exRow = IntArray(image.height)
        var ex = 0
        for (c in exCol.indices) {
            var hasGalaxies = false
            for (r in 0 until image.height) {
                if (image[c, r] == '#') hasGalaxies = true
            }
            if (!hasGalaxies) ex += exp
            exCol[c] = ex
        }
        ex = 0
        for (r in exRow.indices) {
            var hasGalaxies = false
            for (c in 0 until image.width) {
                if (image[c, r] == '#') hasGalaxies = true
            }
            if (!hasGalaxies) ex += exp
            exRow[r] = ex
        }
        val gal = image.findMatches { it == '#' }.map { it.first + exCol[it.first] to it.second + exRow[it.second] }.toTypedArray()
        var minSum = 0L
        var pairCount = 0
        for (g1 in 0 until gal.lastIndex) {
            for (g2 in g1 + 1..gal.lastIndex) {
                val d = abs(gal[g1].first - gal[g2].first) + abs(gal[g1].second - gal[g2].second)
                minSum += d
                pairCount++
            }
        }
        return minSum
    }

    // test if implementation meets criteria from the description, like:
    val testInput = inlineTestInput.trim().reader().readLines()
    //val testInput = readInput("aoc2023/Day11_test")
    val testInputPart1Result = parts(testInput, 1)
    println("Part 1 Test: $testInputPart1Result")
    val testInputPart2aResult = parts(testInput, 9)
    println("Part 2a Test: $testInputPart2aResult")
    val testInputPart2Result = parts(testInput, 99)
    println("Part 2 Test: $testInputPart2Result")
    check(testInputPart1Result == 374L)
    check(testInputPart2aResult == 1030L)
    check(testInputPart2Result == 8410L)

    val input = readInput("aoc2023/Day11")
    parts(input, 1).println()
    parts(input, 1000000 - 1).println()
}
